<?php

class FrmAwbrAppController{
	public static function load_hooks() {
		add_action( 'plugins_loaded', 'FrmAwbrAppController::load_lang' );
		add_action( 'admin_init', 'FrmAwbrAppController::include_updater', 1 );

		// 2.0 hooks
		add_action( 'frm_trigger_aweber_action', 'FrmAwbrAppController::trigger_aweber', 10, 3 );

		// < 2.0 hooks
		add_action( 'frm_entry_form', 'FrmAwbrAppController::hidden_form_fields', 10, 2 );
		add_action( 'frm_after_create_entry', 'FrmAwbrAppController::maybe_send_to_aweber', 25, 2 );
	}
    
    public static function path(){
        return dirname(dirname(__FILE__));
    }

    public static function load_lang(){
        load_plugin_textdomain( 'formidable-aweber', false, self::path() . '/languages/' );
    }

    public static function include_updater(){
		if ( class_exists( 'FrmAddon' ) ) {
			include_once( self::path() . '/models/FrmAwbrUpdate.php' );
			FrmAwbrUpdate::load_hooks();
		}
    }
    
    public static function hidden_form_fields($form, $form_action){
        $form->options = maybe_unserialize($form->options);
        if(!isset($form->options['aweber']) or !$form->options['aweber'] or !isset($form->options['awbr_list']) or !is_array($form->options['awbr_list']))
            return;
            
        echo '<input type="hidden" name="frm_aweber" value="1"/>'."\n";
        
        //if($form_action != 'update')
            return;
        
        $list = reset($form->options['awbr_list']);
        $field_id = $list['fields']['email'];
        
        global $frm_editing_entry, $frm_vars;

		$entry_id = ( is_array( $frm_vars ) && isset( $frm_vars['editing_entry'] ) ) ? $frm_vars['editing_entry'] : $frm_editing_entry;
		$email = FrmEntryMeta::get_entry_meta( absint( $entry_id ), $field_id );
        
        echo '<input type="hidden" name="frm_aweber_email" value="'. esc_attr($email) .'"/>'."\n";
    }

	public static function trigger_aweber( $action, $entry, $form ) {
		$settings = $action->post_content;
		self::send_to_aweber( $entry, $form, $settings );
	}

	public static function maybe_send_to_aweber( $entry_id, $form_id ) {
		if ( ! isset( $_POST ) || ! isset( $_POST['frm_aweber'] ) ) {
			return;
		}
        
		$form = FrmForm::getOne( $form_id );
        
		if ( ! isset( $form->options['aweber'] ) || ! $form->options['aweber'] ) {
            return;
		}
        
        foreach($form->options['awbr_list'] as $list_id => $list_options){
            //check conditions
            $subscribe = true;
			if ( isset( $list_options['hide_field'] ) && is_array( $list_options['hide_field'] ) ) {
                //for now we are assuming that if all conditions are met, then the user will be subscribed
				foreach ( $list_options['hide_field'] as $hide_key => $hide_field ) {
					if ( ! $subscribe ) {
						continue;
					}
                        
					$observed_value = ( isset( $_POST['item_meta'][ $hide_field ] ) ) ? sanitize_text_field( $_POST['item_meta'][ $hide_field ] ) : '';
                    
                    if ($observed_value == ''){
                        $subscribe = false;
                    }else{
                        $subscribe = FrmProFieldsHelper::value_meets_condition($observed_value, $list_options['hide_field_cond'][$hide_key], $list_options['hide_opt'][$hide_key]);
                    }
                }
            }

			if ( ! $subscribe ) {
				//don't subscribe if conditional logic is not met
				continue;
			}

            $list_options['list_id'] = $list_id;
            self::send_to_aweber( $entry, $form, $list_options );
        }
    }

	public static function send_to_aweber( $entry, $form, $list_options ) {
		$list = FrmAwbrAppHelper::get_aweber_list( $list_options['list_id'] );
		if ( empty( $list ) ) {
			return;
		}

		$vars = array(
			'custom_fields' => array(),
			'ip_address' => $_SERVER['REMOTE_ADDR']
		);

		foreach ( $list_options['fields'] as $field_tag => $field_id ) {
			$val = FrmAwbrAppHelper::get_entry_or_post_value( $entry, $field_id );
			if ( is_numeric( $val ) ) {
				$field = FrmField::getOne( $field_id );
				if ( $field->type == 'user_id' ) {
					$user_data = get_userdata( $val );
					if ( $field_tag == 'email' ) {
						$val = $user_data->user_email;
					} else if ( $field_tag == 'name' ) {
						$val = $user_data->first_name .' '. $user_data->last_name;
					} else {
						$val = $user_info->user_login;
					}
				} else {
					$value_atts = array( 'type' => $field->type, 'truncate' => false, 'entry_id' => $entry_id ) ;
					if ( is_callable('FrmEntriesHelper::display_value') ) {
						$val = FrmEntriesHelper::display_value( $val, $field, $value_atts );
					} else if ( is_callable('FrmProEntryMetaHelper::display_value') ) {
						$val = FrmProEntryMetaHelper::display_value( $val, $field, $value_atts );
					}
				}
			}

			if ( $field_tag == 'email' || $field_tag == 'name' ) {
				$vars[ $field_tag ] = $val;
			} else {
				$vars['custom_fields'][ $field_tag ] = $val;
			}
		}

		$vars = apply_filters( 'frm_awbr_vars', $vars, $form );

		if ( ! isset( $vars['email'] ) ) {
			//no email address is mapped
			return;
		}

		if ( empty( $vars['custom_fields'] ) ) {
			unset( $vars['custom_fields'] );
		}

		try{
			$subscribers = $list->subscribers;
			$new_subscriber = $subscribers->create( $vars );
		}
		catch ( FrmAWeberAPIException $exc ) {
		}
	}
}